// ModelSwarm.java

import swarm.Globals;
import swarm.Selector;
import swarm.defobj.Zone;

import swarm.objectbase.Swarm;
import swarm.objectbase.SwarmImpl;
import swarm.objectbase.EmptyProbeMap;
import swarm.objectbase.EmptyProbeMapImpl;

import swarm.activity.ActionGroupImpl;
import swarm.activity.ScheduleImpl;
import swarm.activity.Activity;

import swarm.collections.ListImpl;
import swarm.collections.ListIndex;
import swarm.collections.ListShufflerImpl;
import swarm.collections.ArrayImpl;

/**
 * The Model of JavaSum. The Model contains the Units
 * and all the related tools.
 *
 * @author Marco Agagliate  
 * 
 */
public class ModelSwarm extends SwarmImpl
        {
        // Declare the model parameters and their default values.
    
        public int randomAgentNumber                = 300;
        public int marketImitatingAgentNumber       = 0;
        public int locallyImitatingAgentNumber      = 0;
        public int stopLossAgentNumber              = 0;
        /** The total number of agents. */
        public int  agentNumber                     = randomAgentNumber+marketImitatingAgentNumber+
	    locallyImitatingAgentNumber+stopLossAgentNumber;
        /** The total number of instants per day. */
        public int  instantNumber                   = 300;
        /** The number of current day. */
        public int dayNumber                        = 0;
        /** The asimmetry of buy and sell probability. */
        public double asymmetricBuySellProb         = 0.9;
        /** The coefficient that RandomAgents multiplays
         *  by the last price for the order.
         */
        public double  minCorrectingCoeff           = 0.95;
        /** The coefficient that RandomAgents multiplays
         *  by the last price for the order.
         */
        public double  maxCorrectingCoeff           = 1.05;
        /**The asimmetry of max/minCorrectingCoeff.  */
        public double  asymmetricRange              = 0.0;
        /** The probability of placing an order with market price. */
        public double  agentProbToActWithMarketPrice  = 0.0;
        /** The probability of placing an order in the opening or closing phase.
         *  So a day starts and finish with an action, with a realistic effect. 
         */
        public double  agentProbToActInOpeningOrClosingAuctions  = 0.5;
        /** The minimum price at which the agents acts. */
        public double  floorP                       = 0.3;
        /** The probability to act below floor price. */
        public double  agentProbToActBelowFloorP    = 0.5;
        /** The maximum number of order per agent. */
        public int maxOrderQuantity                 = 3;

        /** The max rate of loss that stop loss agents effort. */
        public double maxLossRate                   = 0.1;
        /** The interval that stop loss agents consider for their strategy. */
        public int stopLossInterval                 = 2;
        /** To check the position(sharequantity) of the agent. */
        public int checkingIfShortOrLong            = 1;

        /**  The percentage quota of agents which act in the market.*/
        public int percentageOfOperatingAgents      = 0;
        /**  The maximum number of operating agents. */
        public int maxNumberOfOperatingAgents       = 1;
        /**  The type of percetage of agents. If it is true percentage
         *   is a maximum value; if it is false percentage is a
         *   fixed value.
         */
        public boolean typeOfPercentage             = false;

        /** If 1 book prints data on the terminal window,
          * 2 for Agent. */
        public int printing                         = 0;  
                                                           
        // Declare some other needed variables.
        /** The list of all the agents. */
        public ListImpl agentList;

        /** The list of the RandomAgents. */
        public ListImpl randomAgentList;
        /** The list of the MarketImitatingAgents.*/
        public ListImpl marketImitatingAgentList;
        /** The list of the LocallyImitatingAgents.*/
        public ListImpl locallyImitatingAgentList;
        /** The list of the StopLossAgents.*/
        public ListImpl stopLossAgentList;

        /** The list of the agents in create order. */
        public ListImpl indexAgentList;

        /** Its iterator. */
        public ListIndex indexAgentListIndex;

        /** ActionGroup for holding an ordered sequence of action. */
        public ActionGroupImpl  modelActions1,
                                modelActionsLS,
                                modelActions2,
                                modelActions3,
                                modelActions4;

        /** The Schedule operating in the Model. */
        public ScheduleImpl modelSchedule;

        /** The book of the simulation. */
        public Book theBook;
        /** The agents of the simulation. */
        public RandomAgent              anAgent1;
        public MarketImitatingAgent     anAgent2;
        public LocallyImitatingAgent    anAgent3;
        public StopLossAgent            anAgent4;
       
        /** The randomRuleMaster manages RandomAgents. */
        public RandomRuleMaster randomRuleMaster;
        /** The stopLossRuleMaster manages StopLossAgents.*/
        public StopLossRuleMaster stopLossRuleMaster;
        /** This is a "ghost agent".
         *  See the comments of CurrentAgent class.
         */
        public CurrentAgent theCurrentAgent;
        /**  The object for the calls of the agents. */
        public CurrentInstant theCurrentInstant;



        /** Constructor for a new ModelSwarm. */  
        public ModelSwarm(Zone aZone)
                {
                // Use the parent class to create a top-level swarm.
                super(aZone);

                // Build a customized probe map.  Without a custom probe map
                // the default is to show all variables and messages.  Here we
                // choose to customize the appearance of the probe, giving a
                // nicer interface.

                // Create the probe map and give it the ModelSwarm class.
                EmptyProbeMapImpl probeMap = new EmptyProbeMapImpl(aZone, getClass());
   
                // Now add probes for the variables we wish to probe.
                probeMap.addProbe(Globals.env.probeLibrary
                        .getProbeForVariable$inClass("randomAgentNumber",
                                ModelSwarm.this.getClass ()));
                probeMap.addProbe(Globals.env.probeLibrary
                        .getProbeForVariable$inClass("marketImitatingAgentNumber",
                                ModelSwarm.this.getClass ()));
                probeMap.addProbe(Globals.env.probeLibrary
                        .getProbeForVariable$inClass("locallyImitatingAgentNumber",
                                ModelSwarm.this.getClass ()));
                probeMap.addProbe(Globals.env.probeLibrary
                        .getProbeForVariable$inClass("stopLossAgentNumber",
                                ModelSwarm.this.getClass ()));
                probeMap.addProbe(Globals.env.probeLibrary
                        .getProbeForVariable$inClass("agentNumber",
                                ModelSwarm.this.getClass ()));
                probeMap.addProbe(Globals.env.probeLibrary
                        .getProbeForVariable$inClass("instantNumber",
                                ModelSwarm.this.getClass ()));
                probeMap.addProbe(Globals.env.probeLibrary
                        .getProbeForVariable$inClass("dayNumber",
                                ModelSwarm.this.getClass ()));
                probeMap.addProbe(Globals.env.probeLibrary
                        .getProbeForVariable$inClass("asymmetricBuySellProb",
                                ModelSwarm.this.getClass ()));
                probeMap.addProbe(Globals.env.probeLibrary
                        .getProbeForVariable$inClass("minCorrectingCoeff",
                                ModelSwarm.this.getClass ()));
                probeMap.addProbe(Globals.env.probeLibrary
                        .getProbeForVariable$inClass("maxCorrectingCoeff",
                                ModelSwarm.this.getClass ()));
                probeMap.addProbe(Globals.env.probeLibrary
                        .getProbeForVariable$inClass("asymmetricRange",
                                ModelSwarm.this.getClass ()));
                probeMap.addProbe(Globals.env.probeLibrary
                        .getProbeForVariable$inClass("agentProbToActWithMarketPrice",
                                ModelSwarm.this.getClass ()));
                probeMap.addProbe(Globals.env.probeLibrary
                        .getProbeForVariable$inClass("agentProbToActInOpeningOrClosingAuctions",
                                ModelSwarm.this.getClass ()));
                probeMap.addProbe(Globals.env.probeLibrary
                        .getProbeForVariable$inClass("floorP",
                                ModelSwarm.this.getClass ()));
                probeMap.addProbe(Globals.env.probeLibrary
                        .getProbeForVariable$inClass("agentProbToActBelowFloorP",
                                ModelSwarm.this.getClass ()));
                probeMap.addProbe(Globals.env.probeLibrary
                        .getProbeForVariable$inClass("maxOrderQuantity",
                                ModelSwarm.this.getClass ()));
                probeMap.addProbe(Globals.env.probeLibrary
                        .getProbeForVariable$inClass("maxLossRate",
                                ModelSwarm.this.getClass ()));
                probeMap.addProbe(Globals.env.probeLibrary
                        .getProbeForVariable$inClass("stopLossInterval",
                                ModelSwarm.this.getClass ()));
                probeMap.addProbe(Globals.env.probeLibrary
                        .getProbeForVariable$inClass("checkingIfShortOrLong",
                                ModelSwarm.this.getClass ()));
                probeMap.addProbe(Globals.env.probeLibrary
                        .getProbeForVariable$inClass("percentageOfOperatingAgents",
                                ModelSwarm.this.getClass ()));
                probeMap.addProbe(Globals.env.probeLibrary
                        .getProbeForVariable$inClass("maxNumberOfOperatingAgents",
                                ModelSwarm.this.getClass ()));
                probeMap.addProbe(Globals.env.probeLibrary
                        .getProbeForVariable$inClass("typeOfPercentage",
                                ModelSwarm.this.getClass ()));
                probeMap.addProbe(Globals.env.probeLibrary
                        .getProbeForVariable$inClass("printing",
                                ModelSwarm.this.getClass ())); 
                probeMap.addProbe(Globals.env.probeLibrary
                        .getProbeForMessage$inClass("openProbeTo:",
                                ModelSwarm.this.getClass ()));

                // And finaly install our probe map into the probeLibrary.
                // Note that this library was created by initSwarm().
                Globals.env.probeLibrary.setProbeMap$For(probeMap, getClass());
                }

        /** Build the model objects. */
        public Object buildObjects()
                {
                int i;
                
                // use the parent class buildObject() method to initialize the
                // process
                super.buildObjects();
                
                // Now create the model's objects.
                // randomRuleMaster
                randomRuleMaster = new RandomRuleMaster(getZone());
                randomRuleMaster.setAgentProbToActInOpeningOrClosingAuctions
                                        ( agentProbToActInOpeningOrClosingAuctions );
                randomRuleMaster.setMinCorrectingCoeff( minCorrectingCoeff );
                randomRuleMaster.setMaxCorrectingCoeff( maxCorrectingCoeff );
                randomRuleMaster.setAsymmetricRange( asymmetricRange );
                randomRuleMaster.setFloorP$andAgentProbToActBelowFloorP
                                        ( floorP, agentProbToActBelowFloorP );

                // stopLossRuleMaster
                stopLossRuleMaster = new StopLossRuleMaster(getZone());
                stopLossRuleMaster.setAgentProbToActInOpeningOrClosingAuctions
                                        ( agentProbToActInOpeningOrClosingAuctions );
                stopLossRuleMaster.setMinCorrectingCoeff( minCorrectingCoeff );
                stopLossRuleMaster.setMaxCorrectingCoeff( maxCorrectingCoeff );
                stopLossRuleMaster.setAsymmetricRange( asymmetricRange );
                stopLossRuleMaster.setFloorP$andAgentProbToActBelowFloorP
                                        ( floorP, agentProbToActBelowFloorP );
 

                // Now create a List object to manage all the agent we are
                // about to create.
                agentList =                 new ListImpl(getZone());
                randomAgentList =           new ListImpl(getZone());
                marketImitatingAgentList =  new ListImpl(getZone());
                locallyImitatingAgentList = new ListImpl(getZone());
                stopLossAgentList =         new ListImpl(getZone());
                indexAgentList =            new ListImpl(getZone());
                indexAgentListIndex =       indexAgentList.listBegin(getZone());

                
                if (agentNumber==0)
                        {
                        System.out.println("Nonsense: agentNumber cannot be 0");
                        System.exit(1);
                        }

		// the book
                theBook = new Book(getZone());
                theBook.setAgentListIndex( indexAgentListIndex );
		theBook.setPrinting( printing );
		theBook.setInstantNumber( instantNumber );
		if(printing==1){
                  System.out.println("Length of meanPriceHistory: " +theBook.meanPriceHistory.length);            
		}
      
                // a few checks
                if (asymmetricBuySellProb<0.5)
                        {
                        System.out.println
                        ("The asymmetricBuySellProb cannot be < 0.5 (internally set to 0.5).\n");
                        asymmetricBuySellProb=0.5;
                        }

                // randomAgent
                for (i=1;i<=randomAgentNumber;i++)
                        {
                        anAgent1 = new RandomAgent(getZone(), maxOrderQuantity);
                        anAgent1.setNumber( i );
                        anAgent1.setMaxOrderQuantity( maxOrderQuantity );
                        anAgent1.setAgentProbToActWithMarketPrice( agentProbToActWithMarketPrice );
                        anAgent1.setBook( theBook );
                        anAgent1.setRuleMaster( randomRuleMaster );
                        anAgent1.setPrinting( printing );
                        agentList.addLast(anAgent1);
                        randomAgentList.addLast(anAgent1);
                        indexAgentList.addLast(anAgent1);
                        }

                // marketImitatingAgent
                for (i=randomAgentNumber+1;i<=randomAgentNumber+marketImitatingAgentNumber;i++)
                        {
                        anAgent2 = new MarketImitatingAgent(getZone(), maxOrderQuantity);
                        anAgent2.setNumber( i );
                        anAgent2.setMaxOrderQuantity( maxOrderQuantity );
                        anAgent2.setAgentProbToActWithMarketPrice( agentProbToActWithMarketPrice );
                        anAgent2.setAsymmetricBuySellProb( asymmetricBuySellProb );
                        anAgent2.setBook( theBook );
                        anAgent2.setRuleMaster( randomRuleMaster );                        
                        anAgent2.setPrinting( printing );
                        agentList.addLast(anAgent2);
                        marketImitatingAgentList.addLast(anAgent2);
                        indexAgentList.addLast(anAgent2);
                        }

                // locallyImitatingAgent
                for (i=randomAgentNumber+marketImitatingAgentNumber+1;i<=randomAgentNumber+marketImitatingAgentNumber+locallyImitatingAgentNumber;i++)
                        {
                        anAgent3 = new LocallyImitatingAgent(getZone(), maxOrderQuantity);
                        anAgent3.setNumber( i );
                        anAgent3.setMaxOrderQuantity( maxOrderQuantity );
                        anAgent3.setAgentProbToActWithMarketPrice( agentProbToActWithMarketPrice );
                        anAgent3.setAsymmetricBuySellProb( asymmetricBuySellProb );
                        anAgent3.setBook( theBook );
                        anAgent3.setRuleMaster( randomRuleMaster );                        
                        anAgent3.setPrinting( printing );
                        agentList.addLast(anAgent3);
                        locallyImitatingAgentList.addLast(anAgent3);
                        indexAgentList.addLast(anAgent3);
                        }

                // stopLossAgent
                for (i=randomAgentNumber+marketImitatingAgentNumber+locallyImitatingAgentNumber+1;i<=randomAgentNumber+marketImitatingAgentNumber+locallyImitatingAgentNumber+stopLossAgentNumber;i++)
                        {
                        anAgent4 = new StopLossAgent(getZone(), maxOrderQuantity);
                        anAgent4.setNumber( i );
                        anAgent4.setMaxOrderQuantity( maxOrderQuantity );
                        anAgent4.setAgentProbToActWithMarketPrice( agentProbToActWithMarketPrice );
                        anAgent4.setStopLossInterval( stopLossInterval );
                        anAgent4.setMaxLossRate$andCheckingIfShortOrLong( maxLossRate,checkingIfShortOrLong );
                        anAgent4.setBook( theBook );
                        anAgent4.setRuleMaster( stopLossRuleMaster );                        
                        anAgent4.setPrinting( printing );
                        agentList.addLast(anAgent4);
                        stopLossAgentList.addLast(anAgent4);
                        indexAgentList.addLast(anAgent4);
                        }

                // current agent
                theCurrentAgent = new CurrentAgent(getZone());
                theCurrentAgent.setAgentList( agentList );
		theCurrentAgent.setPrinting( printing );

                // number of operating agents
                               
                   maxNumberOfOperatingAgents
                        = percentageOfOperatingAgents*agentNumber/100;
                   if (percentageOfOperatingAgents == 0)
                           maxNumberOfOperatingAgents = 1;

                // current instant
                theCurrentInstant = new CurrentInstant(getZone());
                theCurrentInstant.setCurrentAgent( theCurrentAgent );
                theCurrentInstant.setMaxNumberOfOperatingAgents
                                        ( maxNumberOfOperatingAgents );
                theCurrentInstant.setPercentageOfOperatingAgents
                                        ( percentageOfOperatingAgents );
                theCurrentInstant.setTypeOfPercentage( typeOfPercentage );
		theCurrentInstant.setPrinting( printing );
                return this;
                }


        /**
         * Here is where the model schedule is built, the data structures
         * that define the simulation of time in the model. The core is an
         * actionGroup that has a list of actions. Then that's put in a Schedule.
         */
        public Object buildActions()
                {
                Selector sel;
                
                int i, j;

                ListShufflerImpl listShuffler = new ListShufflerImpl(getZone());

                // First, use the parent class to initialize the process.
                super.buildActions();
                
                // We create the list of simulation actions

		//Open auction
                modelActions1 = new ActionGroupImpl(getZone());
                sel = SwarmUtils.getSelector(this, "setAgentNumber");
                modelActions1.createActionTo$message(this, sel);
                sel = SwarmUtils.getSelector("Book", "setClean");
                modelActions1.createActionTo$message(theBook, sel);
                sel = SwarmUtils.getSelector(listShuffler, "shuffleWholeList");
                modelActions1.createActionTo$message(listShuffler, sel, agentList);
                sel = SwarmUtils.getSelector("BasicSumAgent", "act0");
                modelActions1.createActionForEach$message(agentList, sel);
                sel = SwarmUtils.getSelector("Book", "setTheoreticalPrice");
                modelActions1.createActionTo$message(theBook, sel);
                sel = SwarmUtils.getSelector("Book", "setOpeningPrice");
                modelActions1.createActionTo$message(theBook, sel);
                sel = SwarmUtils.getSelector("Book", "opening");
                modelActions1.createActionTo$message(theBook, sel);

                //Shuffle list
                modelActionsLS = new ActionGroupImpl(getZone());
                sel = SwarmUtils.getSelector(listShuffler, "shuffleWholeList");
                modelActionsLS.createActionTo$message(listShuffler, sel, agentList);

                //Acting in the market
                modelActions2 = new ActionGroupImpl(getZone());
                sel = SwarmUtils.getSelector("CurrentInstant", "callAgents");
                modelActions2.createActionTo$message(theCurrentInstant, sel);
                sel = SwarmUtils.getSelector("Book", "setNumberOfInstant");
                modelActions2.createActionTo$message(theBook, sel);
                
		//Close auction 
                modelActions3 = new ActionGroupImpl(getZone());
                sel = SwarmUtils.getSelector("Book", "recreateTableForTheoreticalPrice");
                modelActions3.createActionTo$message(theBook, sel); 
                sel = SwarmUtils.getSelector(listShuffler, "shuffleWholeList");
                modelActions3.createActionTo$message(listShuffler, sel, agentList);
                sel = SwarmUtils.getSelector("BasicSumAgent", "act0");
                modelActions3.createActionForEach$message(agentList, sel);
                sel = SwarmUtils.getSelector("Book", "setTheoreticalPrice");
                modelActions3.createActionTo$message(theBook, sel);
                sel = SwarmUtils.getSelector("Book", "setOpeningPrice");
                modelActions3.createActionTo$message(theBook, sel);
                sel = SwarmUtils.getSelector("Book", "opening");
                modelActions3.createActionTo$message(theBook, sel);               

                //Accounting
                modelActions4 = new ActionGroupImpl(getZone());
                sel = SwarmUtils.getSelector("Book", "setMeanPrice");
                modelActions4.createActionTo$message(theBook, sel);
                sel = SwarmUtils.getSelector("BasicSumAgent", "act2");
                modelActions4.createActionForEach$message(agentList, sel);
                sel = SwarmUtils.getSelector(this, "increaseCurrentDayNumber");
                modelActions4.createActionTo$message(this, sel);
                
                // Then we create a schedule that executes the modelActions.
                // We use here agentNumber steps in each cycle, while 
                // the observer uses a low display frequency (e.g. 1) 
                // to show the price of each step-tick we can also use a high d.f.
                // (e.g. 1000 with 100 agents) to run a faster simulation.

                modelSchedule = new ScheduleImpl(getZone(), instantNumber);
                modelSchedule.at$createAction(0, modelActions1);

                for (i=0;i<instantNumber;i++)
                        {
                        modelSchedule.at$createAction(i, modelActionsLS);
                        modelSchedule.at$createAction(i, modelActions2);
                        }
                modelSchedule.at$createAction(instantNumber-1, modelActions3);
                modelSchedule.at$createAction(instantNumber-1, modelActions4);
                
                return this;
                }

        /**
         * Now set up the model's activation. swarmContext indicates where
         * we're being started in - typically, this model is run as a
         * subswarm of an observer swarm. */
        public Activity activateIn(Swarm swarmContext)
                {
                // Use the parent class to activate ourselves in the context
                // passed to us.
                super.activateIn(swarmContext);

                // Then activate the schedule in ourselves.
                modelSchedule.activateIn(this);

                // Finally, return the activity we have built.
                return getActivity();
                }

        /** The method increases the number of the day. */
        public void increaseCurrentDayNumber()
                {
                dayNumber++;
                if(printing==1)
                        System.out.println("Day number " + dayNumber);
                }

        /** The method set the number of the agents. */
        public void setAgentNumber()
                {
	         agentNumber= randomAgentNumber+marketImitatingAgentNumber+
                              locallyImitatingAgentNumber+stopLossAgentNumber;
                }


        /** The method returns the number of the day. */
        public int getCurrentDay()
                {
                return dayNumber;
                }

        /** The method returns the list of the agents. */
        public ListImpl getAgentList()
                {
                return agentList;
                }

        /** The method returns the list of the RandomAgents. */
        public ListImpl getRandomAgentList()
                {
                return randomAgentList;
                }

        /** The method returns the list of the MarketImitatingAgents. */
        public ListImpl getMarketImitatingAgentList()
                {
                return marketImitatingAgentList;
                }

        /** The method returns the list of the LocallyImitatingAgents. */
        public ListImpl getLocallyImitatingAgentList()
                {
                return locallyImitatingAgentList;
                }

        /** The method returns the list of the StopLossAgents. */
        public ListImpl getStopLossAgentList()
                {
                return stopLossAgentList;
                }


        /** The method returns the index of indexAgentList. */
        public ListIndex getAgentListIndex()
                {
                return indexAgentListIndex;
                }

        /** The method returns the book. */
        public Book getBook()
                {
                return theBook;
                }

        /** The method opens the probe on an agent. */
        public void openProbeTo(int n)
                {
                BasicSumAgent anAgent;
                
                if (n>=1 && n<=agentNumber)
                        {
                        indexAgentListIndex.setOffset(n-1);
                        anAgent = (BasicSumAgent)indexAgentListIndex.get();
                        Globals.env.createArchivedProbeDisplay(anAgent, "anAgent");
                        }
                }
        }
